import os
import csv

import datetime

import arcpy

# Author J.Connolly USEPA
# Description:  Projects files into the standard NAD 83 geographic projection before the files are moved to
# the species spatial libraries. STD geo projection is NAD83.


# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.


# in gdb from step 5 and 6
InGDB = r"path\filename.gdb"

# Location of prj files
proj_Folder = r'path\folder'

# Temp GDB for geo transformations
middlegdb = r'path\middle_gdb.gdb'

# Workspace for copied and standardize files; stays static for the whole tool
ws = "path"
# Folder in workspace where outputs will be saved
name_dir = "folder name"
# in yyyymmdd received date
receivedDate = 'date'

# file names for outputs
ReProjectGDB = "gdb files_name.gdb"
CSVName = "csv file name.csv"

NADabb = 'NAD83'
Unknowncsv = 'Unknown.csv'

# Geo projections that have been see previously NOTE if a new project turns up then it will  need to be added to the
# projection folder and added to this script
coordFile = proj_Folder + os.sep + 'NAD_1983.prj'
WGScoordFile = proj_Folder + os.sep + 'WGS_1984.prj'
Nad83HarncoordFile = proj_Folder + os.sep + 'NAD 1983 HARN.prj'


# ################################################################################################################


# Functions
def fcs_in_workspace(workspace):
    # recursively checks workspaces found within the inFileLocation and makes list of all feature class
    arcpy.env.workspace = workspace
    for fcs in arcpy.ListFeatureClasses():
        yield (fcs)
    for wks in arcpy.ListWorkspaces():
        for fcs in fcs_in_workspace(wks):
            yield (fcs)


def create_directory(path_directory, out_csv, out_gdb):
    # creates directories to save files
    if not os.path.exists(path_directory):
        os.mkdir(path_directory)
        print "created directory {0}".format(path_directory)
    if not os.path.exists(out_csv):
        os.mkdir(out_csv)
        print "created directory {0}".format(out_csv)
    if not os.path.exists(out_gdb):
        os.mkdir(out_gdb)
        print "created directory {0}".format(out_gdb)


def create_flnm_timestamp(namefile, outlocation, date_list, file_extension):
    # creates date stamped generic file
    file_extension.replace('.', '')
    filename = str(namefile) + "_" + str(date_list[0]) + '.' + file_extension
    filepath = os.path.join(outlocation, filename)
    return filename, filepath


def create_out_table(list_name, csv_name):
    # outputs table from list generated in create FileList
    with open(csv_name, "wb") as output:
        writer = csv.writer(output, lineterminator='\n')
        for val in list_name:
            writer.writerow([val])


def create_gdb(out_folder, out_name, outpath):
    # Create a new GDB
    if not arcpy.Exists(outpath):
        arcpy.CreateFileGDB_management(out_folder, out_name, "CURRENT")


# Static variable no user input needed #####################################################################
# Creates file names

datelist = []
todaydate = datetime.date.today()
datelist.append(todaydate)

path_dir = ws + os.sep + str(name_dir)
outLocationCSV = path_dir + os.sep + "CSV"
OutFolderGDB = path_dir + os.sep + "GDB"

NADCSV = str(NADabb) + "_" + str(CSVName)
Unknwn = str(Unknowncsv) + "_" + str(CSVName)

GeoReProject, geocsvfile = create_flnm_timestamp(NADCSV, outLocationCSV, datelist, 'csv')
# CSV out table succeed file-complete spatial package

UnknownProject, unknowncsv = create_flnm_timestamp(Unknwn, outLocationCSV, datelist, 'csv')
FNADabb = "Failed_" + str(NADabb)

NADFailed, NADfailedcsv = create_flnm_timestamp(FNADabb, outLocationCSV, datelist, 'csv')

otherGeoprjGDB = str(NADabb) + "_" + str(ReProjectGDB)

otherGeoGDB, OtherGeoGDBpath = create_flnm_timestamp(otherGeoprjGDB, OutFolderGDB, datelist, 'gdb')

outWorkspaceOtherGeo = OtherGeoGDBpath

# NOTE Change this to False if you don't want GDB to be overwritten
arcpy.env.overwriteOutput = True
arcpy.env.workspace = OutFolderGDB

# Empty list to store information about each of the feature classes

OrgGeoList = []
AddGeoList = "Filename" + "," + "Starting Projection Name" + "," + "Starting Projection Type" + "," + \
             "Filename Reprojected" + "," + "Final Geographic System Name" + "," + "Middle Geographic System Name"
OrgGeoList.append(AddGeoList)

FailedGeoList = []
addFailedGeo = "Filename-(GDB)"
FailedGeoList.append(addFailedGeo)

FailPrjList = []
addFailedPrj = "Filename-(GDB)"
FailPrjList.append(addFailedPrj)

OrgPrjList = []
AddPrjList = "Filename" + "," + "Re-project Projected System Name"
OrgPrjList.append(AddPrjList)

UnknownList = []
addUnknown = "Filename-(GDB)", ",", "Projection"
UnknownList.append(addUnknown)

# #############################################################################################################
# start clock on timing script

start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

create_directory(path_dir, outLocationCSV, OutFolderGDB)

create_gdb(OutFolderGDB, otherGeoGDB, OtherGeoGDBpath)
outgdb = OtherGeoGDBpath

# Extracts spatial information from prj files
dsc = arcpy.Describe(coordFile)
coord_sys = dsc.spatialReference

dscwgs = arcpy.Describe(WGScoordFile)
wgscoord_sys = dscwgs.spatialReference

dscnadharn = arcpy.Describe(Nad83HarncoordFile)
nad83scoord_sys = dscnadharn.spatialReference

grouplist = []
NADabb = "NAD83"

unknownprj = []
group_gdb = InGDB

arcpy.env.workspace = group_gdb
fc_list = arcpy.ListFeatureClasses()


# for each file in in GDB checks the spatial reference, and re-projects into the standard,
# if a transformation is needed this is done to the temp gdb
for fc in fc_list:
    infc = group_gdb + os.sep + fc
    ORGdsc = arcpy.Describe(fc)
    ORGsr = ORGdsc.spatialReference
    ORGprj = ORGsr.name.lower()
    if ORGdsc.spatialReference.Type == "Projected":
        print "Projected Conversion"
        # print ORGsr.GCS.datumName
        if ORGsr.GCS.datumName == "D_North_American_1983":
            fcNAD = str(fc) + "_" + str(NADabb)
            outfc = outgdb + os.sep + fcNAD
            if arcpy.Exists(outfc):
                print str(fc) + " already exists"
                continue
            else:
                arcpy.Project_management(infc, outfc, coord_sys)
                print "completed {0}".format(fc)

        elif ORGsr.GCS.datumName == "D_WGS_1984":
            fcotherGEO = str(fc) + "_WGS84"
            outotherfc = middlegdb + os.sep + fcotherGEO

            fcNAD = str(fcotherGEO) + "_" + str(NADabb)
            outfc = outgdb + os.sep + fcNAD

            if not arcpy.Exists(outotherfc):
                arcpy.Project_management(infc, outotherfc, wgscoord_sys)
            if not arcpy.Exists(outfc):
                arcpy.Project_management(outotherfc, outfc, coord_sys)
                print "completed {0}".format(fc)
            else:
                print str(fc) + " already exists"
        elif ORGsr.GCS.datumName == 'D_North_American_1983_HARN':
            fcotherGEO = str(fc) + "_NAD83HARN"
            outotherfc = middlegdb + os.sep + fcotherGEO

            fcNAD = str(fcotherGEO) + "_" + str(NADabb)
            outfc = outgdb + os.sep + fcNAD

            if not arcpy.Exists(outotherfc):
                arcpy.Project_management(infc, outotherfc, nad83scoord_sys)
            if not arcpy.Exists(outfc):
                arcpy.Project_management(outotherfc, outfc, coord_sys)
                print "completed {0}".format(fc)
            else:
                print str(fc) + " already exists"
        else:
            print ORGsr.name.lower()
            print ORGsr.GCS.datumName
            unknownprj.append(fc)
            print "Unknown projection"

    elif ORGdsc.spatialReference.Type == "Geographic":
        print "Geographic Conversion"
        if ORGsr.GCS.datumName == "D_WGS_1984":
            fcotherGEO = str(fc) + "_WGS84"
            outfc = outgdb + os.sep + fcotherGEO + "_" + str(NADabb)

            if not arcpy.Exists(outfc):
                arcpy.Project_management(infc, outfc, coord_sys)
                print "completed {0}".format(fc)
            else:
                print str(fc) + " already exists"

        elif ORGsr.GCS.datumName == "D_North_American_1983":
            fcNAD = str(fc) + "_" + str(NADabb)
            outfc = outgdb + os.sep + fcNAD
            if not arcpy.Exists(outfc):
                arcpy.Project_management(fc, outfc, coord_sys)
                print "completed {0}".format(fc)
            else:
                print str(fc) + " already exists"

        elif ORGsr.GCS.datumName == 'D_North_American_1983_HARN':
            fcotherGEO = str(fc) + "_NAD83HARN"
            outotherfc = middlegdb + os.sep + fcotherGEO

            fcNAD = str(fcotherGEO) + "_" + str(NADabb)
            outfc = outgdb + os.sep + fcNAD

            if not arcpy.Exists(outotherfc):
                arcpy.Project_management(infc, outotherfc, nad83scoord_sys)
            if not arcpy.Exists(outfc):
                arcpy.Project_management(outotherfc, outfc, coord_sys)
                print "completed {0}".format(fc)
            else:
                print str(fc) + " already exists"
        elif ORGsr.GCS.datumName == 'North_American_Datum_1983':
            fcotherGEO = str(fc) + "_NAD83"
            outfc = outgdb + os.sep + fcotherGEO

            if not arcpy.Exists(outfc):
                arcpy.Project_management(infc, outfc, coord_sys)
                print "completed {0}".format(fc)

            else:
                print str(fc) + " already exists"

        else:
            print ORGsr.name.lower()
            print ORGsr.GCS.datumName
            unknownprj.append(fc)
            print "Failed to add filename " + str(fc)

    else:
        unknownprj.append(fc)
        print "Failed to add filename " + str(fc)

print "Failed fcs {0}".format(unknownprj)

create_out_table(unknownprj, outLocationCSV + os.sep + 'Unknown_projection.csv')

# #End clock time script
end = datetime.datetime.now()
print "End Time: " + end.ctime()

elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
